@@ -7,7 +7,7 @@
 
 /*
   Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
-  Copyright (C) 2004-2006, 2009 Peter Simonsson <peter.simonsson@gmail.com>
+  Copyright (C) 2004-2016 Peter Simonsson <peter.simonsson@gmail.com>
   Copyright (C) 2006-2008 Eike Hein <hein@kde.org>
 */
 
@@ -28,6 +28,7 @@
 #include "notificationhandler.h"
 #include "viewcontainer.h"
 
+#include <QRegExp>
 #include <QSplitter>
 #include <QTimer>
 #include <QToolButton>
@@ -207,7 +208,7 @@ Channel::Channel(QWidget* parent, const QString& _name) : ChatWindow(parent)
     nickListButtons = new QFrame(m_horizSplitter);
     m_horizSplitter->setStretchFactor(m_horizSplitter->indexOf(nickListButtons), 0);
     QVBoxLayout* nickListButtonsLayout = new QVBoxLayout(nickListButtons);
-    nickListButtonsLayout->setSpacing(spacing());
+    nickListButtonsLayout->setSpacing(0);
     nickListButtonsLayout->setMargin(0);
 
     nicknameListView=new NickListView(nickListButtons, this);
@@ -529,8 +530,19 @@ void Channel::completeNick()
         m_inputBar->setOldCursorPosition(pos);
         // remember old cursor position locally
         oldPos = pos;
-        // step back to last space or start of line
-        while(pos && line[pos-1] != QLatin1Char(' ')) pos--;
+        // step back to []{}-_^`\| or start of line
+        QString regexpStr("[^A-Z0-9a-z\\_\\[\\]\\{\\}\\-\\^\\`\\\\\\|");
+
+        if(!Preferences::self()->prefixCharacter().isEmpty())
+            regexpStr += "\\" + Preferences::self()->prefixCharacter();
+
+        regexpStr += QLatin1Char(']');
+        QRegExp tmp(regexpStr);
+        pos = tmp.lastIndexIn(line, pos - 1);
+        if (pos < 0)
+            pos = 0;
+        else
+            pos++;
         // copy search pattern (lowercase)
         QString pattern = line.mid(pos, oldPos - pos);
         // copy line to newLine-buffer
@@ -1045,7 +1057,7 @@ bool Channel::shouldShowEvent(ChannelNickPtr channelNick)
         return true; // if hideUnimportantEvents is off we don't care and just show the event
 }
 
-void Channel::nickRenamed(const QString &oldNick, const NickInfo& nickInfo)
+void Channel::nickRenamed(const QString &oldNick, const NickInfo& nickInfo, const QHash<QString, QString> &messageTags)
 {
     QString newNick = nickInfo.getNickname();
     Nick *nick = getNickByName(oldNick);
@@ -1063,12 +1075,12 @@ void Channel::nickRenamed(const QString &oldNick, const NickInfo& nickInfo)
     {
         setNickname(newNick);
         if (displayCommandMessage)
-            appendCommandMessage(i18n("Nick"),i18n("You are now known as %1.", newNick), true, true);
+            appendCommandMessage(i18n("Nick"),i18n("You are now known as %1.", newNick), messageTags, true, true);
     }
     else if (displayCommandMessage)
     {
         /* No, must've been someone else */
-        appendCommandMessage(i18n("Nick"),i18n("%1 is now known as %2.", oldNick, newNick));
+        appendCommandMessage(i18n("Nick"),i18n("%1 is now known as %2.", oldNick, newNick), messageTags);
     }
 
     if (nick)
@@ -1080,7 +1092,7 @@ void Channel::nickRenamed(const QString &oldNick, const NickInfo& nickInfo)
     }
 }
 
-void Channel::joinNickname(ChannelNickPtr channelNick)
+void Channel::joinNickname(ChannelNickPtr channelNick, const QHash<QString, QString> &messageTags)
 {
     bool displayCommandMessage = shouldShowEvent(channelNick);
 
@@ -1090,7 +1102,7 @@ void Channel::joinNickname(ChannelNickPtr channelNick)
         emit joined(this);
         if (displayCommandMessage)
             appendCommandMessage(i18nc("Message type", "Join"), i18nc("%1 = our hostmask, %2 = channel",
-                                 "You (%1) have joined the channel %2.", channelNick->getHostmask(), getName()), false, true);
+                                 "You (%1) have joined the channel %2.", channelNick->getHostmask(), getName()), messageTags, false, true);
 
         // Prepare for impending NAMES.
         purgeNicks();
@@ -1118,12 +1130,12 @@ void Channel::joinNickname(ChannelNickPtr channelNick)
         QString hostname = channelNick->getHostmask();
         if (displayCommandMessage)
             appendCommandMessage(i18nc("Message type", "Join"), i18nc("%1 is the nick joining and %2 the hostmask of that nick",
-                                 "%1 (%2) has joined this channel.", nick, hostname), false);
+                                 "%1 (%2) has joined this channel.", nick, hostname), messageTags, false);
         addNickname(channelNick);
     }
 }
 
-void Channel::removeNick(ChannelNickPtr channelNick, const QString &reason, bool quit)
+void Channel::removeNick(ChannelNickPtr channelNick, const QString &reason, bool quit, const QHash<QString, QString> &messageTags)
 {
     bool displayCommandMessage = shouldShowEvent(channelNick);
 
@@ -1144,18 +1156,18 @@ void Channel::removeNick(ChannelNickPtr channelNick, const QString &reason, bool
             if (quit)
             {
                 if (displayReason.isEmpty())
-                    appendCommandMessage(i18nc("Message type", "Quit"), i18n("You (%1) have left this server.", channelNick->getHostmask()));
+                    appendCommandMessage(i18nc("Message type", "Quit"), i18n("You (%1) have left this server.", channelNick->getHostmask()), messageTags);
                 else
                     appendCommandMessage(i18nc("Message type", "Quit"), i18nc("%1 = our hostmask, %2 = reason", "You (%1) have left this server (%2).",
-                        channelNick->getHostmask(), displayReason), false);
+                        channelNick->getHostmask(), displayReason), messageTags, false);
             }
             else
             {
                 if (displayReason.isEmpty())
-                    appendCommandMessage(i18nc("Message type", "Part"), i18n("You have left channel %1.", getName()));
+                    appendCommandMessage(i18nc("Message type", "Part"), i18n("You have left channel %1.", getName()), messageTags);
                 else
                     appendCommandMessage(i18nc("Message type", "Part"), i18nc("%1 = our hostmask, %2 = channel, %3 = reason",
-                        "You (%1) have left channel %2 (%3).", channelNick->getHostmask(), getName(), displayReason), false);
+                        "You (%1) have left channel %2 (%3).", channelNick->getHostmask(), getName(), displayReason), messageTags, false);
             }
         }
 
@@ -1169,19 +1181,19 @@ void Channel::removeNick(ChannelNickPtr channelNick, const QString &reason, bool
             {
                 if (displayReason.isEmpty())
                     appendCommandMessage(i18nc("Message type", "Quit"), i18n("%1 (%2) has left this server.", channelNick->getNickname(),
-                        channelNick->getHostmask()), false);
+                        channelNick->getHostmask()), messageTags, false);
                 else
                     appendCommandMessage(i18nc("Message type", "Quit"), i18nc("%1 = nick, %2 = hostname, %3 = reason",
-                        "%1 (%2) has left this server (%3).", channelNick->getNickname(), channelNick->getHostmask(), displayReason), false);
+                        "%1 (%2) has left this server (%3).", channelNick->getNickname(), channelNick->getHostmask(), displayReason), messageTags, false);
             }
             else
             {
                 if (displayReason.isEmpty())
                     appendCommandMessage(i18nc("Message type", "Part"), i18n("%1 (%2) has left this channel.", channelNick->getNickname(),
-                        channelNick->getHostmask()), false);
+                        channelNick->getHostmask()), messageTags, false);
                 else
                     appendCommandMessage(i18nc("Message type", "Part"), i18nc("%1 = nick, %2 = hostmask, %3 = reason",
-                        "%1 (%2) has left this channel (%3).", channelNick->getNickname(), channelNick->getHostmask(), displayReason), false);
+                        "%1 (%2) has left this channel (%3).", channelNick->getNickname(), channelNick->getHostmask(), displayReason), messageTags, false);
             }
         }
 
@@ -1213,7 +1225,7 @@ void Channel::flushNickQueue()
     processQueuedNicks(true);
 }
 
-void Channel::kickNick(ChannelNickPtr channelNick, const QString &kicker, const QString &reason)
+void Channel::kickNick(ChannelNickPtr channelNick, const QString &kicker, const QString &reason, const QHash<QString, QString> &messageTags)
 {
     QString displayReason = reason;
 
@@ -1229,22 +1241,22 @@ void Channel::kickNick(ChannelNickPtr channelNick, const QString &kicker, const
         if(kicker == m_server->getNickname())
         {
             if (displayReason.isEmpty())
-                appendCommandMessage(i18n("Kick"), i18n("You have kicked yourself from channel %1.", getName()));
+                appendCommandMessage(i18n("Kick"), i18n("You have kicked yourself from channel %1.", getName()), messageTags);
             else
                 appendCommandMessage(i18n("Kick"), i18nc("%1 adds the channel and %2 the reason",
-                                              "You have kicked yourself from channel %1 (%2).", getName(), displayReason));
+                                              "You have kicked yourself from channel %1 (%2).", getName(), displayReason), messageTags);
         }
         else
         {
             if (displayReason.isEmpty())
             {
                 appendCommandMessage(i18n("Kick"), i18nc("%1 adds the channel, %2 adds the kicker",
-                                              "You have been kicked from channel %1 by %2.", getName(), kicker));
+                                              "You have been kicked from channel %1 by %2.", getName(), kicker), messageTags);
             }
             else
             {
                 appendCommandMessage(i18n("Kick"), i18nc("%1 adds the channel, %2 the kicker and %3 the reason",
-                                              "You have been kicked from channel %1 by %2 (%3).", getName(), kicker, displayReason));
+                                              "You have been kicked from channel %1 by %2 (%3).", getName(), kicker, displayReason), messageTags);
             }
 
             Application::instance()->notificationHandler()->kick(this,getName(), kicker);
@@ -1264,22 +1276,22 @@ void Channel::kickNick(ChannelNickPtr channelNick, const QString &kicker, const
         if(kicker == m_server->getNickname())
         {
             if (displayReason.isEmpty())
-                appendCommandMessage(i18n("Kick"), i18n("You have kicked %1 from the channel.", channelNick->getNickname()));
+                appendCommandMessage(i18n("Kick"), i18n("You have kicked %1 from the channel.", channelNick->getNickname()), messageTags);
             else
                 appendCommandMessage(i18n("Kick"), i18nc("%1 adds the kicked nick and %2 the reason",
-                                     "You have kicked %1 from the channel (%2).", channelNick->getNickname(), displayReason));
+                                     "You have kicked %1 from the channel (%2).", channelNick->getNickname(), displayReason), messageTags);
         }
         else
         {
             if (displayReason.isEmpty())
             {
                 appendCommandMessage(i18n("Kick"), i18nc("%1 adds the kicked nick, %2 adds the kicker",
-                                     "%1 has been kicked from the channel by %2.", channelNick->getNickname(), kicker));
+                                     "%1 has been kicked from the channel by %2.", channelNick->getNickname(), kicker), messageTags);
             }
             else
             {
                 appendCommandMessage(i18n("Kick"), i18nc("%1 adds the kicked nick, %2 the kicker and %3 the reason",
-                                     "%1 has been kicked from the channel by %2 (%3).", channelNick->getNickname(), kicker, displayReason));
+                                     "%1 has been kicked from the channel by %2 (%3).", channelNick->getNickname(), kicker, displayReason), messageTags);
             }
         }
 
@@ -1357,7 +1369,7 @@ QString Channel::getTopic()
     return m_topicHistory->currentTopic();
 }
 
-void Channel::setTopic(const QString& text)
+void Channel::setTopic(const QString& text, const QHash<QString, QString> &messageTags)
 {
     QString cleanTopic = text;
 
@@ -1365,12 +1377,12 @@ void Channel::setTopic(const QString& text)
     if (!cleanTopic.isEmpty() && hasIRCMarkups(cleanTopic))
         cleanTopic += QStringLiteral("\017");
 
-    appendCommandMessage(i18n("Topic"), i18n("The channel topic is \"%1\".", cleanTopic));
+    appendCommandMessage(i18n("Topic"), i18n("The channel topic is \"%1\".", cleanTopic), messageTags);
 
     m_topicHistory->appendTopic(replaceIRCMarkups(Konversation::removeIrcMarkup(text)));
 }
 
-void Channel::setTopic(const QString& nickname, const QString& text)
+void Channel::setTopic(const QString& nickname, const QString& text, const QHash<QString, QString> &messageTags)
 {
     QString cleanTopic = text;
 
@@ -1379,9 +1391,9 @@ void Channel::setTopic(const QString& nickname, const QString& text)
         cleanTopic += QStringLiteral("\017");
 
     if (nickname == m_server->getNickname())
-        appendCommandMessage(i18n("Topic"), i18n("You set the channel topic to \"%1\".", cleanTopic));
+        appendCommandMessage(i18n("Topic"), i18n("You set the channel topic to \"%1\".", cleanTopic), messageTags);
     else
-        appendCommandMessage(i18n("Topic"), i18n("%1 sets the channel topic to \"%2\".", nickname, cleanTopic));
+        appendCommandMessage(i18n("Topic"), i18n("%1 sets the channel topic to \"%2\".", nickname, cleanTopic), messageTags);
 
     m_topicHistory->appendTopic(replaceIRCMarkups(Konversation::removeIrcMarkup(text)), nickname);
 }
@@ -1394,7 +1406,7 @@ void Channel::setTopicAuthor(const QString& author, QDateTime time)
     m_topicHistory->setCurrentTopicMetadata(author, time);
 }
 
-void Channel::updateMode(const QString& sourceNick, char mode, bool plus, const QString &parameter)
+void Channel::updateMode(const QString& sourceNick, char mode, bool plus, const QString &parameter, const QHash<QString, QString> &messageTags)
 {
     // Note for future expansion:
     //     m_server->getChannelNick(getName(), sourceNick);
@@ -1826,7 +1838,7 @@ void Channel::updateMode(const QString& sourceNick, char mode, bool plus, const
 
     if (!message.isEmpty() && !Preferences::self()->useLiteralModes())
     {
-        appendCommandMessage(i18n("Mode"), message);
+        appendCommandMessage(i18n("Mode"), message, messageTags);
     }
 
     updateModeWidgets(mode, plus, parameter);
@@ -1910,13 +1922,10 @@ void Channel::updateModeWidgets(char mode, bool plus, const QString &parameter)
     emit modesChanged();
 }
 
-void Channel::updateQuickButtons(const QStringList &newButtonList)
+void Channel::updateQuickButtons()
 {
-    // remove quick buttons from memory and GUI
-    qDeleteAll(buttonList);
-    buttonList.clear();
-
     delete m_buttonsGrid;
+    m_buttonsGrid = 0;
 
     // the grid that holds the quick action buttons
     m_buttonsGrid = new QWidget (nickListButtons); //Q3Grid(2, nickListButtons);
@@ -1927,6 +1936,9 @@ void Channel::updateQuickButtons(const QStringList &newButtonList)
 
     int col = 0;
     int row = 0;
+
+    const QStringList &newButtonList = Preferences::quickButtonList();
+
     // add new quick buttons
     for(int index=0;index<newButtonList.count();index++)
     {
@@ -1935,7 +1947,6 @@ void Channel::updateQuickButtons(const QStringList &newButtonList)
         col = index % 2;
         layout->addWidget (quickButton, row, col);
         row += col;
-        buttonList.append(quickButton);
 
         connect(quickButton, SIGNAL(clicked(QString)), this, SLOT(quickButtonClicked(QString)));
 
@@ -2125,7 +2136,7 @@ void Channel::updateAppearance()
     showTopic(Preferences::self()->showTopic());
     setAutoUserhost(Preferences::self()->autoUserhost());
 
-    updateQuickButtons(Preferences::quickButtonList());
+    QMetaObject::invokeMethod(this, "updateQuickButtons", Qt::QueuedConnection);
 
     // Nick sorting settings might have changed. Trigger timer
     if (m_delayedSortTimer)
@@ -2172,6 +2183,12 @@ void Channel::endOfNames()
     {
         m_initialNamesReceived = true;
 
+        if (m_server->hasAwayNotify() && !Preferences::self()->autoWhoContinuousEnabled())
+        {
+            // Do one who request to get the initial away state for the channel
+            QMetaObject::invokeMethod(m_server, "requestWho", Qt::QueuedConnection, Q_ARG(QString, getName()));
+        }
+
         scheduleAutoWho();
     }
 }
@@ -2490,6 +2507,19 @@ void Channel::processQueuedNicks(bool flush)
         while (nickname.isEmpty() && !m_nickQueue.isEmpty())
             nickname = m_nickQueue.takeFirst();
 
+        QString userHost;
+
+        if(m_server->hasUserHostInNames())
+        {
+            int index = nickname.indexOf(QLatin1Char('!'));
+
+            if(index >= 0)
+            {
+                userHost = nickname.mid(index + 1);
+                nickname = nickname.left(index);
+            }
+        }
+
         bool admin = false;
         bool owner = false;
         bool op = false;
@@ -2513,6 +2543,11 @@ void Channel::processQueuedNicks(bool flush)
             Q_ASSERT(nick);
             nick->setMode(mode);
 
+            if(!userHost.isEmpty())
+            {
+                nick->getNickInfo()->setHostmask(userHost);
+            }
+
             fastAddNickname(nick);
 
             ++m_processedNicksCount;
@@ -2678,7 +2713,7 @@ void Channel::clearBanList()
   emit banListCleared();
 }
 
-void Channel::append(const QString& nickname, const QString& message)
+void Channel::append(const QString& nickname, const QString& message, const QHash<QString, QString> &messageTags, const QString& label)
 {
     if(nickname != getServer()->getNickname()) {
         Nick* nick = getNickByName(nickname);
@@ -2688,11 +2723,11 @@ void Channel::append(const QString& nickname, const QString& message)
         }
     }
 
-    ChatWindow::append(nickname, message);
+    ChatWindow::append(nickname, message, messageTags, label);
     nickActive(nickname);
 }
 
-void Channel::appendAction(const QString& nickname, const QString& message)
+void Channel::appendAction(const QString& nickname, const QString& message, const QHash<QString, QString> &messageTags)
 {
     if(nickname != getServer()->getNickname()) {
         Nick* nick = getNickByName(nickname);
@@ -2702,7 +2737,7 @@ void Channel::appendAction(const QString& nickname, const QString& message)
         }
     }
 
-    ChatWindow::appendAction(nickname, message);
+    ChatWindow::appendAction(nickname, message, messageTags);
     nickActive(nickname);
 }
 
