@@ -8,7 +8,7 @@
 /*
   Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
   Copyright (C) 2005 Ismail Donmez <ismail@kde.org>
-  Copyright (C) 2005-2006 Peter Simonsson <psn@linux.se>
+  Copyright (C) 2005-2016 Peter Simonsson <peter.simonsson@gmail.com>
   Copyright (C) 2006-2008 Eli J. MacKenzie <argonel at gmail.com>
   Copyright (C) 2005-2008 Eike Hein <hein@kde.org>
 */
@@ -138,11 +138,12 @@ class Server : public QObject
         void resetNickSelection();
         void queueNicks(const QString& channelName, const QStringList& nicknameList);
         void addHostmaskToNick(const QString &sourceNick, const QString &sourceHostmask);
-        Channel* nickJoinsChannel(const QString &channelName, const QString &nickname, const QString &hostmask);
-        void renameNick(const QString &nickname,const QString &newNick);
-        Channel* removeNickFromChannel(const QString &channelName, const QString &nickname, const QString &reason, bool quit=false);
-        void nickWasKickedFromChannel(const QString &channelName, const QString &nickname, const QString &kicker, const QString &reason);
-        void removeNickFromServer(const QString &nickname, const QString &reason);
+        Channel* nickJoinsChannel(const QString &channelName, const QString &nickname, const QString &hostmask, const QString &account,
+                                  const QString &realName, const QHash<QString, QString> &messageTags);
+        void renameNick(const QString &nickname, const QString &newNick, const QHash<QString, QString> &messageTags);
+        Channel* removeNickFromChannel(const QString &channelName, const QString &nickname, const QString &reason, const QHash<QString, QString> &messageTags, bool quit=false);
+        void nickWasKickedFromChannel(const QString &channelName, const QString &nickname, const QString &kicker, const QString &reason, const QHash<QString, QString> &messageTags);
+        void removeNickFromServer(const QString &nickname, const QString &reason, const QHash<QString, QString> &messageTags);
 
         void setChannelTypes(const QString &types);
         QString getChannelTypes() const;
@@ -155,6 +156,8 @@ class Server : public QObject
         QString banAddressListModes() { return m_banAddressListModes; }     // aka "TYPE A" modes http://tools.ietf.org/html/draft-brocklesby-irc-isupport-03#section-3.3
 
         void setPrefixes(const QString &modes, const QString& prefixes);
+        QString getServerNickPrefixes() const;
+
         void mangleNicknameWithModes(QString &nickname,bool& isAdmin,bool& isOwner,bool &isOp,
             bool& isHalfop,bool &hasVoice);
 
@@ -167,15 +170,15 @@ class Server : public QObject
         QString getNextNickname();
 
         InputFilter* getInputFilter() { return &m_inputFilter; }
-        Konversation::OutputFilter* getOutputFilter() { return m_outputFilter; };
+        Konversation::OutputFilter* getOutputFilter() { return m_outputFilter; }
 
-        Channel* joinChannel(const QString& name, const QString& hostmask);
+        Channel* joinChannel(const QString& name, const QString& hostmask, const QHash<QString, QString> &messageTags);
         void removeChannel(Channel* channel);
-        void appendServerMessageToChannel(const QString& channel, const QString& type, const QString& message);
-        void appendCommandMessageToChannel(const QString& channel, const QString& command, const QString& message,
+        void appendServerMessageToChannel(const QString& channel, const QString& type, const QString& message, const QHash<QString, QString> &messageTags);
+        void appendCommandMessageToChannel(const QString& channel, const QString& command, const QString& message, const QHash<QString, QString> &messageTags,
                                            bool highlight = true, bool parseURL = true);
-        void appendStatusMessage(const QString& type,const QString& message);
-        void appendMessageToFrontmost(const QString& type,const QString& message, bool parseURL = true);
+        void appendStatusMessage(const QString& type,const QString& message, const QHash<QString, QString> &messageTags);
+        void appendMessageToFrontmost(const QString& type,const QString& message, const QHash<QString, QString> &messageTags = QHash<QString, QString>(), bool parseURL = true);
 
         int getPreLength(const QString& command, const QString& dest);
 
@@ -184,10 +187,10 @@ class Server : public QObject
         void dbusInfo(const QString& string);
         void ctcpReply(const QString& receiver, const QString& text);
 
-        void setChannelTopic(const QString& channel, const QString& topic);
+        void setChannelTopic(const QString& channel, const QString& topic, const QHash<QString, QString> &messageTags);
                                                   // Overloaded
-        void setChannelTopic(const QString& nickname, const QString& channel, const QString& topic);
-        void updateChannelMode(const QString& nick, const QString& channel, char mode, bool plus, const QString& parameter);
+        void setChannelTopic(const QString& nickname, const QString& channel, const QString& topic, const QHash<QString, QString> &messageTags);
+        void updateChannelMode(const QString& nick, const QString& channel, char mode, bool plus, const QString& parameter, const QHash<QString, QString> &messageTags);
         void updateChannelModeWidgets(const QString& channel, char mode, const QString& parameter);
 
         Channel* getChannelByName(const QString& name);
@@ -211,7 +214,7 @@ class Server : public QObject
         QString getOwnIpByServerMessage();
 
         bool isAway() { return m_away; }
-        void setAway(bool away);
+        void setAway(bool away, const QHash<QString, QString> &messageTags);
         QString awayTime() const;
 
         void setAwayReason(const QString& reason) { m_awayReason = reason; }
@@ -383,6 +386,15 @@ class Server : public QObject
         void dccPassiveChatRequest(const QString& recipient, const QString& extension, const QString& address, const QString& token);
         void dccReverseChatAck(const QString& partnerNick, const QString& extension, const QString& ownAddress, quint16 ownPort, const QString& reverseToken);
 
+        bool capEndDelayed() const { return m_capEndDelayed; }
+
+        bool hasAwayNotify() const { return m_hasAwayNotify; }
+        bool hasExtendedJoin() const { return m_hasExtendedJoin; }
+        void setHasWHOX(bool state) { m_hasWHOX = state; }
+        bool hasWHOX() const { return m_hasWHOX; }
+        bool hasServerTime() const { return m_hasServerTime; }
+        bool hasUserHostInNames() const { return m_hasUserHostInNames; }
+
     // IRCQueueManager
         bool validQueue(QueuePriority priority); ///< is this queue index valid?
         void resetQueues(); ///< Tell all of the queues to reset
@@ -421,8 +433,6 @@ class Server : public QObject
          *  @param  encodedBytes    The count of bytes sent to the server after re-encoding.
          */
         void sentStat(int bytes, int encodedBytes, IRCQueue *whichQueue);
-        //FIXME can anyone who can connect to a Server signal not know about an IRCQueue?
-        void sentStat(int bytes, int encodedBytes);
 
         //Note that these signals haven't been implemented yet.
         /// Fires when the information in a NickInfo object changes.
@@ -532,7 +542,8 @@ class Server : public QObject
         /// Will only reconnect if the connection state is involuntary disconnected.
         void reconnectInvoluntary();
 
-        void capInitiateNegotiation();
+        void requestAvailableCapabilies ();
+        void capInitiateNegotiation(const QString &availableCaps);
         void capReply();
         void capEndNegotiation();
         void capCheckIgnored();
@@ -608,8 +619,6 @@ class Server : public QObject
         void requestOpenChannelListPanel(const QString& filter);
 
     private Q_SLOTS:
-        void collectStats(int bytes, int encodedBytes);
-
         /** Called in the server constructor if the preferences are set to run a command on a new server instance.
          *  This sets up the kprocess, runs it, and connects the signals to call preShellCommandExited when done. */
         void doPreShellCommand();
@@ -727,6 +736,7 @@ class Server : public QObject
         QStringList m_inputBuffer;
 
         QList<IRCQueue *> m_queues;
+        // Stats used in QueueTuner
         int m_bytesSent, m_encodedBytesSent, m_linesSent, m_bytesReceived;
 
         QString m_nickname;
@@ -781,6 +791,8 @@ class Server : public QObject
         /// Creates a list of known users and returns the one chosen by the user
         inline QString recipientNick() const;
 
+        void collectStats(int bytes, int encodedBytes);
+
         /// Helper object to construct ISON (notify) list.
         ServerISON* m_serverISON;
         /// All nicks known to this server.  Note this is NOT a list of all nicks on the server.
@@ -822,8 +834,9 @@ class Server : public QObject
         /// Previous ISON reply of the server, needed for comparison with the next reply
         QStringList m_prevISONList;
 
-        bool m_capRequested;
-        bool m_capAnswered;
+        int m_capRequested;
+        int m_capAnswered;
+        bool m_capEndDelayed;
         QString m_lastAuthenticateCommand;
 
         ConnectionSettings m_connectionSettings;
@@ -844,6 +857,12 @@ class Server : public QObject
         QStringList m_changedChannels;
 
         bool m_recreationScheduled;
+
+        bool m_hasAwayNotify;
+        bool m_hasExtendedJoin;
+        bool m_hasWHOX;
+        bool m_hasServerTime;
+        bool m_hasUserHostInNames;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Server::CapModifiers)
